import type { Space, Suffix } from './query';
export declare type Template<K = any, V = any> = Types<K, V> | TemplateObj<K, V>;
export declare type TemplateObj<K = any, V = any> = {
    [key: string]: Template<K, V> | undefined;
};
declare type DelegateParam<T, U, Extra> = T extends Types<infer A, infer B> ? Types<A | Extra, B | U> : T extends object ? {
    [K in keyof T]: T[K] extends Types<infer A, infer B> ? Types<A | Extra, B | U> : Extra extends undefined ? Types<Parse<T[K]> | undefined, U> : DelegateParam<T[K], U, never>;
} : never;
export declare type Parse<T> = T extends Types<infer U, any> ? U : T extends object ? {
    [K in keyof T]: T[K] extends Types<infer U, any> ? U : Parse<T[K]>;
} : never;
export declare type VarParams<T> = T extends AnyType ? VarTypeParams<T> : VarObjectParams<T>;
declare type AnyType = Types<any, any>;
declare type VarTypeParams<T> = T extends Types<any, infer U> ? unknown extends U ? never : U : never;
/**
 * To resolve error: `Type instantiation is excessively deep and possibly infinite`, we should not reference VarParams here, just repeat the type until 5th level.
 *
 * Be careful that 6th level will cause the same error.
 */
declare type VarObjectParams<T> = {
    [K in keyof T]: T[K] extends AnyType ? VarTypeParams<T[K]> : VarObjectParams_2<T[K]>;
}[keyof T];
declare type VarObjectParams_2<T> = {
    [K in keyof T]: T[K] extends AnyType ? VarTypeParams<T[K]> : VarObjectParams_3<T[K]>;
}[keyof T];
declare type VarObjectParams_3<T> = {
    [K in keyof T]: T[K] extends AnyType ? VarTypeParams<T[K]> : VarObjectParams_4<T[K]>;
}[keyof T];
declare type VarObjectParams_4<T> = {
    [K in keyof T]: T[K] extends AnyType ? VarTypeParams<T[K]> : VarObjectParams_5<T[K]>;
}[keyof T];
declare type VarObjectParams_5<T> = {
    [K in keyof T]: T[K] extends AnyType ? VarTypeParams<T[K]> : never;
}[keyof T];
export declare class Types<T = never, U = never> {
    protected fnParams?: string[];
    protected totalParams?: string[];
    protected realName?: string;
    protected includeParam?: string;
    protected skipParam?: string;
    protected returns?: Template;
    /**
     * Set the real server property name.
     * It means current property name is an alias.
     */
    aliasOf(realName: string): this;
    /**
     * Property is a number
     */
    get number(): Types<T | number, U>;
    /**
     * Property is a string
     */
    get string(): Types<T | string, U>;
    /**
     * Property is a bool
     */
    get boolean(): Types<T | boolean, U>;
    /**
     * Property may be undefined
     */
    get undefined(): Types<T | undefined, U>;
    /**
    * Property may be null
    */
    get null(): Types<T | null, U>;
    /**
    * Define custom type:
    * ```typescript
    * types.custom<'User'>();
    *
    * enum Gender {
    *   man = 'man',
    *   woman = 'woman',
    * }
    *
    * types.custom<Gender>();
    * ```
    */
    custom<T1>(): Types<T | T1, U>;
    /**
     * ```
     * {
     *   lists {
     *     id
     *     number
     *   }
     * }
     * ```
     */
    object<T1 extends Template<K, V>, K extends any, V extends any>(items: T1): Types<T | Parse<T1>, U | VarParams<T1>>;
    /**
     * ```
     * {
     *   lists {
     *     id
     *     number
     *   }
     * }
     * ```
     */
    array<T1 extends Template<K, V>, K extends any, V extends any>(each: T1): Types<T | Parse<T1>[], U | VarParams<T1>>;
    /**
     * ```
     * {
     *   id @include (if: $varA)
     *   name
     *   lists (page: 1) @include (if: $varB) {
     *     id
     *     name
     *     age @include (if: $varC)
     *   }
     * }
     * ```
     */
    include<P extends `${string}:${Space}Boolean${Suffix}`>(param_Boolean: P): Types<T | undefined, U | P>;
    /**
     * ```
     * {
     *   id @skip (if: $varA)
     *   name
     *   lists (page: 1) @skip (if: $varB) {
     *     id
     *     name
     *     age @skip (if: $varC)
     *   }
     * }
     * ```
     */
    skip<P extends `${string}:${Space}Boolean${Suffix}`>(param_Boolean: P): Types<T | undefined, U | P>;
    /**
     *
     * @param {String[]} params_Type
     * @example `page: Int` | `name: String!` | `focus: Boolean` | `data: MyObject`
     * @param {Types} returns
     */
    fn<U1 extends `${string}:${string}`, T1 extends Template<K, V>, K extends any, V extends any>(params_Type: U1[], returns: T1): Types<T | Parse<T1>, U | U1 | VarParams<T1>>;
    /**
     * Inline fragment with same struct
     * ```
     * lists {
     *   ... on User {
     *     name
     *   }
     *   ... on Admin {
     *     name
     *   }
     * }
     * ```
     */
    on<T1 extends TemplateObj<K, V>, K extends any, V extends any>(on: string | string[], body: T1): DelegateParam<T1, U, T extends undefined ? undefined : never>;
    /**
     * ```
     * types.union(
     *   types.on('User', {
     *     kind: types.custom<'User'>(),
     *     name: types.string,
     *   }),
     *   types.on('Admin', {
     *     kind: types.custom<'Admin'>(),
     *     counter: types.string,
     *     age: types.number,
     *   }),
     *   ... and more
     * )
     * ```
     */
    union<T1 extends object, T2 extends object>(def1: T1, def2: T2): T1 | T2;
    union<T1 extends object, T2 extends object, T3 extends object>(def1: T1, def2: T2, def3: T3): T1 | T2 | T3;
    union<T1 extends object, T2 extends object, T3 extends object, T4 extends object>(def1: T1, def2: T2, def3: T3, def4: T4): T1 | T2 | T3 | T4;
    protected appendParams(names: string[]): void;
    protected clone(): this;
}
export declare const types: Types<never, never>;
export {};
